local jkHouses = ...

function registerHouseFunction(name, lvl, upgradedToPart)
	local houseStatusList =  {
		{
			DataType = "HOUSE_STATUS",
			RequiredDesirability = DESIRABILITY_LEVEL.LOW,
			ResourceGenerated = lvl,
			HouseDensityPartList = { MOD_NS .. upgradedToPart }
		}
	}
	if lvl == "HOUSING_LEVEL_2" then
		houseStatusList[1].RequiredDesirability = DESIRABILITY_LEVEL.AVERAGE
	end

	local setupId = MOD_NS .. "NON_UPGRADABLE_" .. name .. "_SETUP"
	jkHouses:register({
		DataType= "HOUSE_SETUP",
		Id = setupId,
		HouseStatusList = houseStatusList
	})

	local functionName = MOD_NS .. "NON_UPGRADABLE_" .. name .. "_FUNCTION"
	jkHouses:register({
		DataType = "BUILDING_FUNCTION_HOUSE",
		Id = functionName,
		RelatedZone = "RESIDENTIAL",
		RelatedDesirability = "DESIRABILITY_RESIDENTIAL",
		AssetHouseSetup =  setupId,
		TimeToUpgradeDensity = 1
	})
	return functionName
end

function registerUpgradableHouseFunction(name, upgradedToPart, lvl)
	local lvl = lvl or "HOUSING_LEVEL_1"
	local houseStatusList =  {
		{
			DataType = "HOUSE_STATUS",
			RequiredDesirability = DESIRABILITY_LEVEL.LOW,
			ResourceGenerated = lvl,
			HouseDensityPartList = { MOD_NS .. upgradedToPart }
		},
		{
			DataType = "HOUSE_STATUS",
			RequiredDesirability = DESIRABILITY_LEVEL.AVERAGE,
			ResourceGenerated = lvl,
			HouseDensityPartList = { MOD_NS .. upgradedToPart }
		}
	}
	if lvl == "HOUSING_LEVEL_2" then
		houseStatusList[1].RequiredDesirability = DESIRABILITY_LEVEL.AVERAGE
		houseStatusList[2].RequiredDesirability = DESIRABILITY_LEVEL.VERY_HIGH
	end

	local setupId = MOD_NS .. name .. "_SETUP"
	jkHouses:register({
		DataType= "HOUSE_SETUP",
		Id = setupId,
		HouseStatusList = houseStatusList
	})

	local functionName = MOD_NS .. name .. "_FUNCTION"
	jkHouses:register({
		DataType = "BUILDING_FUNCTION_HOUSE",
		Id = functionName,
		RelatedZone = "RESIDENTIAL",
		RelatedDesirability = "DESIRABILITY_RESIDENTIAL",
		AssetHouseSetup =  setupId,
		TimeToUpgradeDensity = 1
	})
	return functionName
end

function registerStaticPart(fbxPath, name, uiName, category, poly)
    local prefabName = MOD_NS .. "PREFAB_" .. name
    jkHouses:registerAssetId(fbxPath, prefabName)
    local part = {
        DataType = "BUILDING_PART",
        Id = MOD_NS .. name .. "_PART",
        Name = uiName,
        IsMovableWhenBuilt = true,
        ConstructionVisual = prefabName,
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = prefabName
        },
        Cost = {
            BuildRightTaxes = {
                { Resource = "GOLD_COINS", Quantity = 2 }
            },
            RessourcesNeeded = {
                { Resource = "WOOD", Quantity = 5 },  { Resource = "PLANK", Quantity = 1 }
            }
        }
    }

    if category then
        part.Category = category
    end

    if poly then
        part.BuildingZone = { 
            ZoneEntryList = {
                {
                    Polygon = poly,
                    Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
                }
            }
        }
    end

    jkHouses:register(part)
end

function registerMinorStaticPart(fbxPath, name, uiName, category, poly)
    registerStaticPart(fbxPath, name, uiName, category, poly)

    if category == "DOOR" then
        jkHouses:registerPrefabComponent(fbxPath, {
            DataType = "COMP_BUILDING_PART",
            BuildingPartType = ATTACH_NODE_TYPE.DOOR
        })
		jkHouses:registerPrefabComponent(fbxPath, {
            DataType = "COMP_GROUNDED"
        })
        jkHouses:configurePrefabFlagList(fbxPath, { "PLATFORM" })
    else
        jkHouses:registerPrefabComponent(fbxPath, {
            DataType = "COMP_BUILDING_PART",
            BuildingPartType = ATTACH_NODE_TYPE.MINOR
        })
    end
end

function registerSubPart(fbxPath, name)
    local prefabName = MOD_NS .. "PREFAB_HOUSE_EXTENSION_" .. name
    jkHouses:registerAssetId(fbxPath, prefabName)
    jkHouses:register({
        DataType = "BUILDING_PART",
        Id = MOD_NS .. "HOUSE_EXTENSION_" .. name .. "_PART",
        Capacity = 1,
        ConstructionVisual = prefabName,
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = prefabName
        }
    })
end

function registerScalableExtension(prefabPath, name, uiName, size, minScale, functionName)
    local extension = "Extension" .. name
    local part = name .."Part"
    local rootCompFbxPath = prefabPath .. extension .. "/Root" .. part
    registerSubPart(rootCompFbxPath, "ROOT_" .. name)
    registerSubPart(prefabPath  .. extension .. "/Base" .. part, "BASE_" .. name)
    registerSubPart(prefabPath .. extension .. "/Storey" .. part, "STOREY_" .. name)
    registerSubPart(prefabPath  .. extension .. "/Top" .. part, "TOP_" .. name)

    jkHouses:registerPrefabComponent(rootCompFbxPath, {
        DataType = "COMP_DIRT_RECTANGLE",
            Size = size
    })

    local buildingPart = {
        DataType = "BUILDING_PART",
        Id = MOD_NS .. "HOUSE_EXTENSION_" .. name,
		Name = uiName,
        Description = "",
        IsMovableWhenBuilt = true,
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_SCALER",
            CoreObjectPrefab = MOD_NS .. "PREFAB_HOUSE_EXTENSION_ROOT_" .. name,
            EndPart = MOD_NS .."HOUSE_EXTENSION_TOP_" .. name .. "_PART",
            FillerList = {
                MOD_NS .. "HOUSE_EXTENSION_STOREY_" .. name .. "_PART"
            },
            BasementFillerList = {
                MOD_NS .. "HOUSE_EXTENSION_BASE_" .. name .. "_PART"
            },
            MinimumScale = minScale,
            IsVertical = true,
            IsPreviewGroundVisible = true
        },
        BuildingZone = { 
            ZoneEntryList = {
                {
                    Polygon = polygon.createRectangle( size, {0, 0} ),
                    Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
                }
            }
        },
        Cost = {
            RessourcesNeeded = {
                { Resource = "WOOD", Quantity = 5 },
                { Resource = "PLANK", Quantity = 5 },
                { Resource = "STONE", Quantity = 10 }
            }
        }
    }

    if functionName then
        buildingPart.AssetBuildingFunction = functionName
        buildingPart.Category = "CORE"
        jkHouses:registerPrefabComponent(rootCompFbxPath, {
            DataType = "COMP_GROUNDED"
        })
    end

    jkHouses:register(buildingPart)
end

function registerMinorScalableExtensionNoBasement(prefabPath, name, uiName, minScale)
    local extension = "Extension" .. name
    local part = name .."Part"

    registerSubPart(prefabPath .. extension .. "/Root" .. part, "ROOT_" .. name)
    registerSubPart(prefabPath  .. extension .. "/Base" .. part, "BASE_" .. name)
    registerSubPart(prefabPath .. extension .. "/Storey" .. part, "STOREY_" .. name)
    registerSubPart(prefabPath  .. extension .. "/Top" .. part, "TOP_" .. name)

    jkHouses:registerPrefabComponent(prefabPath  .. extension .. "/Base" .. part, {
        DataType = "COMP_BUILDING_PART",
        BuildingPartType = "MINOR"
    })

    jkHouses:register({
        DataType = "BUILDING_PART",
        Id = MOD_NS .. "HOUSE_EXTENSION_" .. name,
        Name = uiName,
        Description = "",
        IsMovableWhenBuilt = true,
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_SCALER",
            CoreObjectPrefab = MOD_NS .. "PREFAB_HOUSE_EXTENSION_BASE_" .. name,
            EndPart = MOD_NS .."HOUSE_EXTENSION_TOP_" .. name .. "_PART",
            FillerList = {
                MOD_NS .. "HOUSE_EXTENSION_STOREY_" .. name .. "_PART"
            },
            MinimumScale = minScale,
            IsVertical = true,
            IsPreviewGroundVisible = false
        },
        BuildingZone = { 
            ZoneEntryList = {
                {
                    Polygon = polygon.createRectangle( {0, 0}, {0, 0} ),
                    Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } 
                }
            }
        },
        Cost = {
            BuildRightTaxes = {
                { Resource = "GOLD_COINS", Quantity = 2 }
            },
            RessourcesNeeded = {
                { Resource = "WOOD", Quantity = 2 }, { Resource = "PLANK", Quantity = 4 }
            }
        }
    })
end

function registerSmokeEmitterPart(houseFbxPath, name)
	local smokeEmitterPrefab = MOD_NS .. "HOUSE_" .. name .. " _SMOKE_EMITTER_PREFAB"
	local emitterPath = houseFbxPath .. "/SmokeEmitter" .. name
	jkHouses:registerAssetId(emitterPath, smokeEmitterPrefab)
	jkHouses:registerPrefabComponent(houseFbxPath, {
		DataType = "COMP_BUILDING_PART",
		FeedbackComponentListToActivate = {
			{
				smokeEmitterPrefab, "COMP_PARTICLE_EMITTER_TOGGLE"
			}
		}
	})
	jkHouses:registerPrefabComponent(emitterPath, {
		DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
		Enabled = false
	})
	jkHouses:registerPrefabComponent(emitterPath, {
		DataType = "COMP_PARTICLE_EMITTER",
		ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
		IsPlaying = false,
		IsEmitting = false
	})
end